import argparse
import ffmpeg
import glob
import os
import sys
import tempfile
import torch
import whisper
from tqdm import tqdm

def setup_environment():
    os.environ["CUDA_LAUNCH_BLOCKING"] = "1"
    print("CUDA GPU is available. Whisper will utilise the GPU for faster processing." if torch.cuda.is_available() else "CUDA GPU not available, falling back to CPU.")

def str2bool(v):
    if isinstance(v, bool):
        return v
    return v.lower() in ('yes', 'true', 't', 'y', '1')

def format_timestamp(seconds):
    hours, remainder = divmod(int(seconds), 3600)
    minutes, seconds = divmod(remainder, 60)
    milliseconds = int((seconds - int(seconds)) * 1000)
    return f"{hours:02}:{minutes:02}:{seconds:02},{milliseconds:03}"

def write_srt(segments, srt_path):
    with open(srt_path, "w", encoding="utf-8") as f:
        for i, segment in enumerate(segments, start=1):
            f.write(f"{i}\n{format_timestamp(segment['start'])} --> {format_timestamp(segment['end'])}\n{segment['text']}\n\n")

def extract_audio(video_path):
    audio_path = os.path.join(tempfile.mkdtemp(), os.path.basename(video_path) + ".wav")
    ffmpeg.input(video_path).output(audio_path, acodec="pcm_s16le", ac=1, ar="16000").run(quiet=True)
    return audio_path

def process_video(video_path, model, language, task):
    srt_path = f"{os.path.splitext(video_path)[0]}.{language}.srt"
    if os.path.exists(srt_path):
        print(f"Skipping {video_path}, SRT file already exists.")
        return

    print(f"Processing: {video_path}")
    audio_path = extract_audio(video_path)
    result = model.transcribe(audio_path, language=language if language != "auto" else None, task=task)
    write_srt(result['segments'], srt_path)
    os.remove(audio_path)
    print(f"Generated SRT: {srt_path}")

def collect_videos(video_dir, videos_arg, language):
    videos_to_process = []
    supported_extensions = ["mp4", "mkv", "ts", "flv", "avi", "mov", "webm", "wmv", "mpg", "m4v"]
    if video_dir:
        for ext in supported_extensions:
            videos_to_process += glob.glob(os.path.join(video_dir, "**", f"*.{ext}"), recursive=True)
    videos_to_process += videos_arg
    # Filter out videos that already have an SRT file for the specified language.
    return [v for v in videos_to_process if not os.path.exists(f"{os.path.splitext(v)[0]}.{language}.srt")]

def main():
    setup_environment()

    parser = argparse.ArgumentParser(description="Transcribe and optionally translate videos using OpenAI Whisper.")
    parser.add_argument("--video_dir", type=str, help="Directory containing video files to process.")
    parser.add_argument("--model", default="base", choices=whisper.available_models(), help="Whisper model to use for transcription. Options are: tiny, base, small, medium, large, large-v2 & large-v3")
    parser.add_argument("--language", type=str, default="en", help="Language code for transcription or translation.")
    parser.add_argument("--task", type=str, default="transcribe", choices=["transcribe", "translate"], help="Task to perform.")
    parser.add_argument("videos", nargs="*", type=str, help="Individual video files to process.")

    args = parser.parse_args()
    model = whisper.load_model(args.model)

    videos_to_process = collect_videos(args.video_dir, args.videos, args.language)

    if not videos_to_process:
        print("No videos found to process.")
        sys.exit(1)

    for video_path in tqdm(videos_to_process, desc="Processing Videos"):
        process_video(video_path, model, args.language, args.task)

if __name__ == "__main__":
    main()

    
    #Options are: tiny, base, small, medium, large, large-v2 & large-v3"